# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import _, api, models, fields
from openerp.addons import decimal_precision as dp
import logging
from lxml import etree

_logger = logging.getLogger(__name__)


class stock_production_lot(models.Model):
    _inherit = 'stock.production.lot'

    @api.model
    def search_read(self, domain=None, fields=None, offset=0, limit=None, order=None):
        context = self._context
        cr = self._cr
        if context.get('pos'):
            # Modify domain
            prod_id = context.get('product_id')
            loc_ids = tuple(context.get('loc_ids'))
            cr.execute("""
            SELECT spl.id lot_id FROM stock_production_lot spl
            JOIN stock_quant sq ON sq.lot_id = spl.id
            JOIN stock_location sl ON sl.id = sq.location_id
            LEFT JOIN stock_move sm ON sq.reservation_id = sm.id
            WHERE sq.product_id=%s AND sq.location_id IN %s AND sl.usage ~ 'internal'
            AND (sq.reservation_id IS NULL OR sm.location_dest_id IN %s)  -- Not reserved or Incoming to our locations
            GROUP BY spl.id HAVING sum(sq.qty)>0;
            """, (prod_id, loc_ids, loc_ids))
            lot_ids = [i[0] for i in cr.fetchall()]
            domain = [("id", "in", lot_ids)]
        res = super(stock_production_lot, self).search_read(domain, fields, offset, limit, order)
        return res

    @api.multi
    def check_stock_lot(self):
        """
        Ensure we have enough qty's available in the lot, if not return 0
        """
        self.ensure_one()
        location_ids = tuple([i for i in self._context.get('loc_ids') if i])
        product_id = self._context.get('product_id')
        self._cr.execute("""
        SELECT COALESCE(SUM(sq.qty), 0) qty FROM stock_production_lot spl
        JOIN stock_quant sq ON sq.lot_id = spl.id
        JOIN stock_location sl ON sl.id = sq.location_id
        WHERE sq.product_id=%s AND sl.usage ~ 'internal' AND spl.id=%s;
        """, (product_id, self.id))
        qty = self._cr.fetchone()[0]
        _logger.info("Qty Available %s [product #%s, locations #%s, lot #%s]" % (qty, product_id, location_ids, self.id))
        return 0 if qty <= 0 else qty


class stock_warehouse(models.Model):
    _inherit = 'stock.warehouse'

    @api.model
    def create(self, vals):
        """
        Create a transit location inside the warehouse
        """
        route_id = self.env.ref('financing.route_transit_to_stock')
        vals.update({
            'route_ids': [(4, route_id.id, False)]
        })
        res = super(stock_warehouse, self).create(vals)
        location_obj = self.env['stock.location']
        sub_locations = [
                {'name': _('Transit'), 'active': True, 'is_transit': True},
        ]
        wh_loc_id = self.env['stock.location'].search([
            ('name','=',_(vals.get('code'))),
            ('location_id','=',self.env.ref('stock.stock_location_locations').id)
        ]).id
        for values in sub_locations:
            loc_vals = {
                'name': values['name'],
                'usage': 'internal',
                'location_id': wh_loc_id,
                'active': values['active'],
                'is_transit': values['is_transit'],
            }
            if vals.get('company_id'):
                loc_vals['company_id'] = vals.get('company_id')
            transit_loc_id = location_obj.create(loc_vals)
        # Add the push rule that when incoming to transit generate a new picking to stock manual op
        push_obj = self.env['stock.location.path']
        push_rules_list = []
        warehouse = res
        from_loc = transit_loc_id
        dest_loc = self.env['stock.location'].search([
            ('name', 'ilike', 'Stock'),
            ('location_id', '=', wh_loc_id)
        ])
        pick_type_id = self.env['stock.picking.type'].search([
            ('warehouse_id', '=', warehouse.id),
            ('code', '=', 'incoming')
        ])
        push_rules_list.append({
            'name': self._format_rulename(warehouse, from_loc, dest_loc),
            'location_from_id': from_loc.id,
            'location_dest_id': dest_loc.id,
            'route_id': route_id.id,
            'auto': 'manual',
            'picking_type_id': pick_type_id.id,
            'active': True,
            'warehouse_id': warehouse.id,
        })
        for push_rule in push_rules_list:
            push_obj.create(vals=push_rule)
        return res


class stock_move(models.Model):
    _inherit = 'stock.move'

    @api.model
    def default_get(self, fields):
        res = super(stock_move, self).default_get(fields)
        dlda = self._context.get('default_location_dest_id_aux')
        if dlda:
            res['location_dest_id'] = dlda
        return res

    @api.model
    def fields_view_get(self, view_id=None, view_type='form', toolbar=False, submenu=False):
        res = super(stock_move, self).fields_view_get(view_id=view_id, view_type=view_type, toolbar=toolbar, submenu=submenu)
        spt_id = self._context.get('default_picking_type_id')
        if spt_id:
            spt = self.env['stock.picking.type'].browse(spt_id)
            poscfg = self.env.user.pos_config
            if view_type == 'form' and poscfg and spt.code == 'internal':
                # On internal moves for pos users move origin is always from their stock location, dest is all available transit locations
                doc = etree.XML(res['arch'])
                suc_transit_ids = self.env['stock.location'].search([
                    ('usage', '=', 'internal'),
                    ('id', '!=', poscfg.stock_location_transit_id.id),
                    ('is_transit', '=', True),
                ]).ids
                for node in doc.xpath("//field[@name='location_id']"):
                    domain = "[('id', '=', %s)]" % poscfg.stock_location_id.id
                    node.set('domain', domain)
                for node in doc.xpath("//field[@name='location_dest_id']"):
                    # the 'Log a note' button is employee only
                    domain = "[('id', 'in', %s)]" % suc_transit_ids
                    node.set('domain', domain)
                res['arch'] = etree.tostring(doc)

        return res


class stock_location(models.Model):
    _inherit = 'stock.location'

    is_transit = fields.Boolean('Is transitory location?')


class stock_picking(models.Model):
    _inherit = 'stock.picking'

    default_dest_location_id = fields.Many2one('stock.location', string='Default Destination', domain=[('is_transit', '=', True)])

    @api.multi
    def action_confirm(self):
        res = super(stock_picking, self.sudo()).action_confirm()
        return res

    @api.model
    def fields_view_get(self, view_id=None, view_type='form', toolbar=False, submenu=False):
        res = super(stock_picking, self).fields_view_get(view_id=view_id, view_type=view_type, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            spt_id = self._context.get('default_picking_type_id', False)
            if not spt_id:
                return res
            sp_id = self._context.get('active_ids')[0]

            # Obtenemos el spt_code
            if spt_id:
                spt_code = self.env['stock.picking.type'].browse(spt_id).code
            elif sp_id:
                spt_code = self.env['stock.picking'].browse(sp_id).picking_type_id.code

            if spt_code == 'internal':
                doc = etree.XML(res['arch'])
                for node in doc.xpath("//field[@name='default_dest_location_id']"):
                    node.set('invisible', '0')
                    node.set('modifiers', '{}')
                res['arch'] = etree.tostring(doc)
        return res
